#!/usr/bin/env python
"""
Copyright 2018 Amazon.com, Inc. or its affiliates.
All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

   http://aws.amazon.com/apache2.0/

or in the "license" file accompanying this file.
This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.

This script orchestrates the enablement and centralization of Macie2 across an enterprise of AWS accounts.
It takes in a list of AWS Account Numbers, iterates through each account and region to enable Macie2.
It creates each account as a Member in the Macie2 Parent account.
It invites and accepts the invite for each Member account.
"""

import boto3
import sys
import time
import argparse
import re

from collections import OrderedDict
from botocore.exceptions import ClientError

def assume_role(aws_account_number, role_name):
    """
    Assumes the provided role in each account and returns a Macie2 client
    :param aws_account_number: AWS Account Number
    :param role_name: 0Role to assume in target account
    :param aws_region: AWS Region for the Client call, not required for IAM calls
    :return: Macie client in the specified AWS Account and Region
    """

    # Beginning the assume role process for account
    sts_client = boto3.client('sts')

    # Get the current partition
    partition = sts_client.get_caller_identity()['Arn'].split(":")[1]

    response = sts_client.assume_role(
        RoleArn='arn:{}:iam::{}:role/{}'.format(
            partition,
            aws_account_number,
            role_name
        ),
        RoleSessionName='EnableMacie'
    )

    # Storing STS credentials
    session = boto3.Session(
        aws_access_key_id=response['Credentials']['AccessKeyId'],
        aws_secret_access_key=response['Credentials']['SecretAccessKey'],
        aws_session_token=response['Credentials']['SessionToken']
    )

    print("Assumed session for {}.".format(
        aws_account_number
    ))

    return session

def accept_invitations(invited_accounts,aws_region):

    for account in invited_accounts:

        child_session = assume_role(account, args.assume_role)
        print(child_session)

        macie_child_client = child_session.client('macie2', region_name=aws_region)

        invitations = macie_child_client.list_invitations()

        invitationId = invitations['invitations'][0]['invitationId']

        print('Accepting Invite for the ID-{invitationId} from the parent account-{parent} '.format(invitationId=invitationId,parent=args.parent_account))

        try:
            macie_child_client.enable_macie(
                                    findingPublishingFrequency='FIFTEEN_MINUTES',
                                    status='ENABLED'

                    )
            print('Enabled Macie in the {account} in {aws_region}'.format(account=account,aws_region=aws_region))

            
        except ClientError as err:
            errormessage=err.response
            print(errormessage)

        try:
            macie_child_client.accept_invitation(
                invitationId=invitationId,
                masterAccount=args.parent_account
                )

            print('Invitations Accepted in the child account - {child_account} recieved from the parent account - {parent_account}'.format(child_account=account,parent_account=args.parent_account))

        except ClientError as err:
            errormessage=err.response
            print(errormessage)




def list_member_accounts(parent_session, aws_region):
    """
    Returns a list of current members linked to the Macie parent account
    :param aws_region: AWS Region of the Macie parent account
    :return: dict of accountId
    """
    member_dict = dict()

    macie_client = parent_session.client('macie2', region_name=aws_region)
    # Need to paginate and iterate over results
    members_list = macie_client.list_members()
    if members_list['members']:
        for members in members_list['members']:
            print(members['accountId'])
            member_dict.update({members['accountId']: members['relationshipStatus']})

    return member_dict

if __name__ == '__main__':

    # Setup command line arguments
    parser = argparse.ArgumentParser(description='Link AWS Accounts to central Macie Account')
    parser.add_argument('--parent_account', type=str, required=True, help="AccountId for Central AWS Account")
    parser.add_argument('input_file', type=argparse.FileType('r'), help='Path to CSV file containing the list of account IDs and S3 resources')
    parser.add_argument('--assume_role', type=str, required=True, help="Role Name to assume in each account")
    parser.add_argument('--enabled_regions', type=str, help="comma separated list of regions to enable Macie. If not specified, all available regions enabled")

    args = parser.parse_args()

    # Validate parent accountId
    if not re.match(r'[0-9]{12}', args.parent_account):
        raise ValueError("Parent AccountId is not valid")

    # Generate dict with account & email information

    aws_account_dict = OrderedDict()

    for acct in args.input_file.readlines():
        split_line = acct.rstrip().split(",")
        if len(split_line) < 2:
            print("Unable to process line: {}".format(acct))
            continue

        if not re.match(r'[0-9]{12}', str(split_line[0])):
            print("Invalid account number {}, skipping".format(split_line[0]))
            continue

        aws_account_dict[split_line[0]] = split_line[1]

    # Getting Macie regions
    session = boto3.session.Session()

    macie_regions = []
    if args.enabled_regions:
        macie_regions = [str(item) for item in args.enabled_regions.split(',')]
        print("Enabling members in these regions: {}".format(macie_regions))
    else:
        macie_regions = session.get_available_regions('macie')
        print("Enabling members in all available Macie regions {}".format(macie_regions))

    # Setting the invitationmessage
    macie_invite_message = 'Account {account} invites you to join Macie2.'.format(account=args.parent_account)
    #use it while sending invitations
    parent_macie_dict = dict() #used at 4 places
    failed_parent_regions = [] #used at 3 places
    

    # Processing Parent account and enabling macie
    parent_session = assume_role(args.parent_account, args.assume_role)
    for aws_region in macie_regions:
        try: 
            macie_client = parent_session.client('macie2', region_name=aws_region)

            macie_client.enable_macie(
                                    findingPublishingFrequency='FIFTEEN_MINUTES',
                                    status='ENABLED'

                    )
            print('Enabled Macie in the {account} in {aws_region}'.format(account=args.parent_account,aws_region=aws_region))

        except ClientError as err:
            errormessage=err.response
            print(errormessage['Error']['Message'])
            if err.response['ResponseMetadata']['HTTPStatusCode'] == 403:
                print("Failed to emable Macie in Parent account for region: {} due to an authentication error.  Either your credentials are not correctly configured or the region is an OptIn region that is not enabled on the parent account.  Skipping {} and attempting to continue").format(aws_region,aws_region)
                failed_parent_regions.append(aws_region)
            else:
                print("Confirming if macie is paused")
                try:
                    macie_client = parent_session.client('macie2', region_name=aws_region)
                    macie_status = macie_client.get_macie_session()['status']
                    if macie_status == 'PAUSED':
                        print('Macie in this account {account} and {aws_region} is paused. So enabling it before checking the members'.format(account=args.parent_account,aws_region=aws_region))
                        macie_client.update_macie_session(
                                    findingPublishingFrequency='FIFTEEN_MINUTES',
                                    status='ENABLED'
                                    )
                except ClientError as err:
                    errormessage=err.response
                    print(errormessage['Error']['Message'])
                    if err.response['ResponseMetadata']['HTTPStatusCode'] == 403:
                        print("Failed to emable Macie in Parent account for region: {} due to an authentication error.  Either your credentials are not correctly configured or the region is an OptIn region that is not enabled on the parent account.  Skipping {} and attempting to continue").format(aws_region,aws_region)


    for failed_region in failed_parent_regions:
        macie_regions.remove(failed_region)

    # Associating member accounts
    failed_accounts = []
    for account in aws_account_dict.keys():
        try:
            parent_session = assume_role(args.parent_account, args.assume_role)

            for aws_region in macie_regions:
                invited_accounts = []
                print('Beginning {account} in {region}'.format(
                    account=account,
                    region=aws_region
                ))

                macie_client = session.client('macie2', region_name=aws_region)

                # List current members
                member_dict = list_member_accounts(parent_session, aws_region)

                if account not in member_dict:
                    try:
                        macie_client = parent_session.client('macie2', region_name=aws_region)
                        macie_client.create_member(
                                            account={
                                                'accountId': account,
                                                'email': aws_account_dict[account]
                                            },
                                            tags={
                                                'name': 'member-{account}'.format(account=account)
                                            }
                                        )
                        print('Added Account {monitored} to member list in Macie2 parent account {parent} for region {region}'.format(
                            monitored=account,
                            parent=args.parent_account,
                            region=aws_region
                        ))

                        print("sleeping for 10 seconds")
                        time.sleep(5)
                        #member_dict = list_member_accounts(parent_session, aws_region)
                        invited_accounts.append(account)
                    except ClientError as err:
                        errormessage=err.response
                        print(errormessage['Error']['Message'])
                        status=macie_client.get_member(id=account)
                        print(status['relationshipStatus'])
                else:

                    print('Account {monitored} is already a member of {parent} in region {region}'.format(
                        monitored=account,
                        parent=args.parent_account,
                        region=aws_region
                    ))

                #creating invitations for new accounts
                if len(invited_accounts) != 0:
                    print('Creating invitations for the list of these accounts - {invited_accounts}'.format(invited_accounts=invited_accounts))
                    try:
                        macie_client = parent_session.client('macie2', region_name=aws_region)
                        macie_client.create_invitations(
                            accountIds=invited_accounts,
                            disableEmailNotification=False,
                            message='Accept the Invitation in the child accounts-{invited_accounts}'.format(invited_accounts=invited_accounts)
                        )


                        print("Accepting the invitation by assuming the role in child account")

                        accept_invitations(invited_accounts,aws_region)
                    except ClientError as err:
                        print('Error while creating the invitations')
                        errormessage=err.response
                else:
                    print("nothing to add")
        except ClientError as e:
            print("Error Processing Account {}".format(account))
            failed_accounts.append({
                account: repr(e)
            })


    if len(failed_accounts) > 0:
        print("---------------------------------------------------------------")
        print("Failed Accounts")
        print("---------------------------------------------------------------")
        for account in failed_accounts:
            print("{}: \n\t{}".format(
                list(account.keys())[0],
                account[list(account.keys())[0]]
            ))
            print("---------------------------------------------------------------")
